@@ -695,7 +695,7 @@
 
 (define anthy-update-prop-label
   (lambda (ac)
-    (let (id (anthy-context-ac-id ac))
+    (let ((id (anthy-context-ac-id ac)))
     (im-update-prop-label "a\tϾ\nϥ⡼\n"))))
   
 
@@ -709,14 +709,12 @@
     ("branch" "ϥ⡼")
     ("leaf" "" "޻" "޻" "prop_anthy_roma")
     ("leaf" "" "" "" "prop_anthy_kana")
-    ("branch" "ϥ⡼")
     ))
 
 (define anthy-update-prop-list
   (lambda (id)
-    (let (tmp)
     (im-update-prop-list id "branch\tϾ\nleaf\t\tҤ餬\tdesc\tprop_anthy_hira\nbranch\t\tR\t޻\tdesc\tprop_anthy_roma\n")
-    )))
+    ))
 
 (register-im
  'anthy
