@@ -21,7 +21,7 @@
 ;; configs
 (define skk-use-candidate-window? #t)
 (define skk-candidate-op-count 0)
-(define skk-use-recursive-learning? #t)
+(define skk-use-recursive-learning? #f)
 ;; key
 (define skk-latin-key
   (lambda (key key-state)
@@ -154,6 +154,25 @@
     (set-car! (nthcdr 0 c) s)))
 ;; state kana head okuri tail candidates nth rk
 
+
+(define skk-find-root-context
+  (lambda (sc)
+    (let ((pc (skk-context-parent-context sc)))
+      (if pc
+	  (skk-find-root-context pc)
+	  sc))))
+
+(define skk-find-descendant-context
+  (lambda (sc)
+    (let ((csc (skk-context-child-context sc)))
+      (if csc
+	  (skk-find-descendant-context csc)
+	  sc))))
+
+(define skk-current-context
+  (lambda (c)
+    (skk-find-descendant-context (context-data c))))
+
 (define skk-flush
   (lambda (sc)
     (rk-flush (skk-context-rk-context sc))
@@ -171,9 +190,10 @@
     (if (null? skk-dic-init)
 	(begin
 	  (set! skk-dic-init #t)
+	  (if skk-use-recursive-learning?
+	   (require "skk-editor.scm"))
 	  (skk-lib-dic-open skk-dic-file-name)
 	  (skk-lib-read-personal-dictionary skk-personal-dic-filename)))
-    (print skk-personal-dic-filename)
     (let ((c 
 	   (copy-list 
 	    '(skk-state-latin #t "" "" "" () () () () () () () ()))))
@@ -223,6 +243,11 @@
      sc
      (skk-context-nth sc))))
 
+(define skk-commit-raw
+  (lambda (sc id key key-state)
+    (if (not (skk-context-parent-context sc))
+	(im-commit-raw id))))
+
 (define skk-commit
   (lambda (sc id str)
     (im-commit id str)))
@@ -285,6 +310,7 @@
 			(skk-context-kana-mode sc)))
       (im-pushback-preedit id preedit-reverse
                            (rk-pending rkc))
+      ;; child context's preedit
       (if csc
 	  (begin
 	    (im-pushback-preedit id preedit-reverse "[")
@@ -292,13 +318,6 @@
 	    (im-pushback-preedit id preedit-reverse "]")
 	    )))))
 
-(define skk-find-root-context
-  (lambda (sc)
-    (let ((pc (skk-context-parent-context sc)))
-      (if pc
-	  (skk-find-root-context pc)
-	  sc))))
-
 (define skk-update-mode
   (lambda (id sc)
     (let ((stat (skk-context-state sc))
@@ -345,7 +364,7 @@
 
 (define skk-proc-state-direct
   (lambda (c key key-state)
-    (let* ((sc (context-data c))
+    (let* ((sc (skk-current-context c))
 	   (id (context-id c))
 	   (key-str (charcode->string (to-lower-char key)))
 	   (rkc (skk-context-rk-context sc))
@@ -363,7 +382,7 @@
        (if (skk-backspace-key key key-state)
 	   (if (not (rk-backspace rkc))
 	       (begin
-		 (im-commit-raw id)
+		 (skk-commit-raw sc id key key-state)
 		 #f)
 	       #f)
 	   #t)
@@ -371,7 +390,7 @@
 	    (control-key-mask key-state)
 	    (= key 32))
 	   (begin
-	     (im-commit-raw id)
+	     (skk-commit-raw sc id key key-state)
 	     #f)
 	   #t)
        (if (and
@@ -389,7 +408,7 @@
 	    (not (string-find (rk-expect rkc) key-str))
 	    (not (rk-pending rkc)))
 	   (begin
-	     (im-commit-raw id)
+	     (skk-commit-raw sc id key key-state)
 	     (skk-flush sc)
 	     (skk-update-preedit id sc)
 	     #f)
@@ -397,7 +416,7 @@
        (if (symbol? key)
 	   (begin
 	     (skk-flush sc)
-	     (im-commit-raw (context-id c))
+	     (skk-commit-raw sc (context-id c) key key-state)
 	     #f)
 	   #t)
        (begin
@@ -417,7 +436,7 @@
 
 (define skk-proc-state-kanji
   (lambda (c key key-state)
-    (let* ((sc (context-data c))
+    (let* ((sc (skk-current-context c))
 	   (id (context-id c))
 	   (rkc (skk-context-rk-context sc))
 	   (stat (skk-context-state sc))
@@ -484,6 +503,34 @@
 	       (skk-begin-conversion sc id)))))
       nil)))
 
+(define skk-setup-child-context
+  (lambda (sc)
+    (let ((csc (skk-context-new)))
+      (skk-context-set-child-context! sc csc)
+      (skk-context-set-parent-context! csc sc)
+      (skk-context-set-state! csc 'skk-state-direct))))
+
+(define skk-check-candidate-window-begin
+  (lambda (sc id)
+    (if
+     (and
+      (not
+       (skk-context-candidate-window sc))
+      skk-use-candidate-window?
+      (> (skk-context-candidate-op-count sc)
+	 skk-candidate-op-count))
+     (begin
+       (skk-context-set-candidate-window! sc #t)
+       (im-begin-candidate
+	id
+	(skk-lib-get-nr-candidates
+	 (skk-make-string (skk-context-head sc)
+			  #t)
+	 (skk-context-okuri-head sc)
+	 (skk-make-string
+	  (skk-context-okuri sc) #t))
+	0)))))
+
 (define skk-change-candidate-index
   (lambda (sc id incr)
     (if incr
@@ -504,34 +551,23 @@
 					    (skk-context-okuri sc) #t))
 					  1)))))
     (if (not (skk-get-current-candidate sc))
-	(skk-context-set-nth! sc 0))
-    ;; Windowɽ򳫻Ϥ뤫
-    (if
-     (and
-      (not
-       (skk-context-candidate-window sc))
-      skk-use-candidate-window?
-      (> (skk-context-candidate-op-count sc)
-	 skk-candidate-op-count))
-     (begin
-       (skk-context-set-candidate-window! sc #t)
-       (im-begin-candidate
-	id
-	(skk-lib-get-nr-candidates
-	 (skk-make-string (skk-context-head sc)
-			  #t)
-	 (skk-context-okuri-head sc)
-	 (skk-make-string
-	  (skk-context-okuri sc) #t))
-	0)))
-    ;;
-    (if (skk-context-candidate-window sc)
-	(im-update-candidate id (skk-context-nth sc)))
+	(begin
+	  (skk-context-set-nth! sc 0)
+	  (if skk-use-recursive-learning?
+	      (skk-setup-child-context sc))))
+    (if (not (skk-context-child-context sc))
+	(begin
+	  ;; Windowɽ򳫻Ϥ뤫
+	  (skk-check-candidate-window-begin sc id)
+	  ;;
+	  (if (skk-context-candidate-window sc)
+	      (im-update-candidate id (skk-context-nth sc)))))
     #f))
 
+
 (define skk-proc-state-converting
   (lambda (c key key-state)
-    (let ((sc (context-data c))
+    (let ((sc (skk-current-context c))
 	  (id (context-id c))
 	  (res ()))
       (and
@@ -544,6 +580,7 @@
        (if (skk-cancel-key key key-state)
 	   (begin
 	     ;; back to kanji mode
+	     ()
 	     (if (skk-context-candidate-window sc)
 		 (im-end-candidate id))
 	     (skk-context-set-state! sc 'skk-state-kanji)
@@ -603,7 +640,7 @@
 
 (define skk-proc-state-okuri
   (lambda (c key key-state)
-    (let* ((sc (context-data c))
+    (let* ((sc (skk-current-context c))
 	   (rkc (skk-context-rk-context sc))
 	   (id (context-id c))	   
 	   (res))
@@ -614,6 +651,19 @@
 	     (skk-context-set-state! sc 'skk-state-kanji)
 	     #f)
 	   #t)
+       (if (skk-backspace-key key key-state)
+	   (begin
+	     (if (not (rk-backspace rkc))
+		 (begin
+		   (print (cdr (skk-context-okuri sc)))
+		   (if (cdr (skk-context-okuri sc))
+		       (skk-context-set-okuri! sc
+			(cdr (skk-context-okuri sc)))
+		       (begin
+			 (skk-context-set-okuri! sc '())
+		       (skk-context-set-state! sc 'skk-state-kanji)))))
+	     #f)
+	   #t)
        (begin
 	 (set! res
 	       (rk-push-key!
@@ -630,33 +680,34 @@
 
 (define skk-proc-state-latin
   (lambda (c key key-state)
-    (let ((sc (context-data c))
+    (let ((sc (skk-current-context c))
 	  (id (context-id c)))
       (if
        (skk-on-key key key-state)
        (begin
 	 (skk-context-set-state! sc 'skk-state-direct)
 	 (skk-update-mode (context-id c) sc))
-       (im-commit-raw (context-id c)))
+       (skk-commit-raw sc (context-id c) key key-state))
       ())))
 
 (define skk-proc-state-wide-latin
   (lambda (c key key-state)
     (let* ((w (jp-wide (charcode->string key)))
 	   (id (context-id c))
-	   (sc (context-data c)))
+	   (sc (skk-current-context c)))
       (if (skk-on-key key key-state)
 	  (begin
 	    (skk-flush sc)
 	    (skk-update-mode id sc))
 	  (if w
 	      (skk-commit sc id w)
-	      (im-commit-raw id)))
+	      (skk-commit-raw sc id key key-state)))
       ())))
 
-(define skk-do-push-key
-  (lambda (c sc key key-state)
-    (let* ((state (skk-context-state sc))
+(define skk-push-key
+  (lambda (c key key-state)
+    (let* ((sc (skk-current-context c))
+	   (state (skk-context-state sc))
 	   (fun)
 	   (res))
       (if (= state 'skk-state-direct)
@@ -678,12 +729,6 @@
        (context-id c) sc)
       )))
 
-(define skk-push-key
-  (lambda (c sc key key-state)
-    (if (skk-context-child-context sc)
-	(skk-push-key c (skk-context-child-context sc) key key-state)
-	(skk-do-push-key c sc key key-state))))
-
 (define skk-init-handler
   (lambda (id arg)
     (let* ((c (find-context id)))
@@ -699,8 +744,8 @@
 
 (define skk-press-key-handler
   (lambda (id key state)
-    (let* ((c (find-context id)))
-      (skk-push-key c (context-data c) key state))))
+    (let ((c (find-context id)))
+      (skk-push-key c key state))))
 
 (define skk-release-key-handler
   (lambda (id key state)
