@@ -240,7 +240,8 @@
  tutcode-key-release-handler
  tutcode-reset-handler
  tutcode-get-candidate-handler
- tutcode-set-candidate-index-handler)
+ tutcode-set-candidate-index-handler
+ nil)
 
 ;;; ơ֥QWERTYܡѡ
 (define tutcode-rule
