@@ -24,11 +24,12 @@ extern LISP sym_t;
  */
 struct skk_cand_array {
   char *okuri;
-  int nr_cands;/* length of cands array*/
+  int nr_cands;/* length of cands array allocated */
   int nr_real_cands;/* length of read from file part */
   char **cands;
 
   int is_used;
+  struct skk_line *line;
 };
 
 struct skk_line {
@@ -271,6 +272,7 @@ find_candidate_array_from_line(struct skk_line *sl, char *okuri,
   ca->nr_cands = 0;
   ca->nr_real_cands = 0;
   ca->okuri = strdup(okuri);
+  ca->line = sl;
   return ca;
 }
 
@@ -349,6 +351,7 @@ compose_line(struct dic_info *di, char *word, char okuri_head, char *entry)
   sl->cands[0].nr_cands = 0;
   sl->cands[0].nr_real_cands = 0;
   sl->cands[0].is_used = 0;
+  sl->cands[0].line = sl;
 
   /* parse */
   compose_line_parts(di, sl, NULL, entry);
@@ -517,17 +520,9 @@ skk_get_nr_candidates(LISP head_, LISP okuri_head_, LISP okuri_)
   return flocons(n);
 }
 
-static LISP
-skk_commit_candidate(LISP head_, LISP okuri_head_,
-		     LISP okuri_, LISP nth_)
+static
+reorder_candidate(struct skk_cand_array *ca, int nth)
 {
-  int nth;
-  struct skk_cand_array *ca;
-
-  nth = get_c_long(nth_);
-
-  ca = find_cand_array_lisp(head_, okuri_head_, okuri_);
-
   if (ca && ca->nr_cands > nth) {
     char *tmp;
     int i;
@@ -537,6 +532,25 @@ skk_commit_candidate(LISP head_, LISP okuri_head_,
     }
     ca->cands[0] = tmp;
   }
+}
+
+static LISP
+skk_commit_candidate(LISP head_, LISP okuri_head_,
+		     LISP okuri_, LISP nth_)
+{
+  int nth;
+  struct skk_cand_array *ca;
+
+  nth = get_c_long(nth_);
+  if (nth == 0) {
+    return NIL;
+  }
+
+  ca = find_cand_array_lisp(head_, okuri_head_, okuri_);
+
+  reorder_candidate(ca, nth);
+  ca->line->need_save = 1;
+
   return NIL;
 }
 
@@ -570,9 +584,52 @@ parse_dic_line(char *line)
   add_line_to_cache(skk_dic, sl);
 }
 
+static void
+write_out_array(FILE *fp, struct skk_cand_array *ca)
+{
+  int i;
+  if (ca->okuri) {
+    fprintf(fp, "[%s/", ca->okuri);
+    for (i = 0; i < ca->nr_real_cands; i++) {
+      fprintf(fp, "%s/", ca->cands[i]);
+    }
+    fprintf(fp, "]/");
+  } else {
+    for (i = 0; i < ca->nr_real_cands; i++) {
+      fprintf(fp, "%s/", ca->cands[i]);
+    }
+  }
+}
+
 static LISP
 skk_lib_save_personal_dictionary(LISP fn_)
 {
+  FILE *fp;
+  char *fn = uim_get_c_string(fn_);
+  struct skk_line *sl;
+
+  fp = fopen(fn, "w");
+  free(fn);
+  if (!fp) {
+    return NIL;
+  }
+
+  for (sl = skk_dic->head.next; sl; sl = sl->next) {
+    struct skk_cand_array *ca;
+    int i;
+    fprintf(fp, "%s", sl->head);
+    if (sl->okuri_head) {
+      fprintf(fp, "%c /", sl->okuri_head);
+    } else {
+      fprintf(fp, " /");
+    }
+    for (i = 0; i < sl->nr_cand_array; i++) {
+      ca = &sl->cands[i];
+      write_out_array(fp, ca);
+    }
+    fprintf(fp, "\n");
+  }
+  fclose(fp);
   return NIL;
 }
 
