@@ -1,3 +1,4 @@
+#include <ctype.h>
 #include <string.h>
 #include <stdlib.h>
 #include "context.h"
@@ -217,6 +218,31 @@ im_commit_raw(LISP id)
   return NIL;
 }
 
+static LISP
+im_get_raw_key_str(LISP key_, LISP key_state_)
+{
+  int key;
+  int key_state = get_c_long(key_state_);
+  unsigned char buf[0];
+  
+  if (FLONUMP(key_)) {
+    key = get_c_long(key_);
+  } else {
+    return NIL;
+  }
+  if ((key_state != 0 && key_state != UKey_Shift) ||
+      key > 255) {
+    return NIL;
+  }
+  
+  buf[0] = key;
+  buf[1] = 0;
+  if (key_state == UKey_Shift) {
+    buf[0] = toupper(buf[0]);
+  }
+  return strcons(1, buf);
+}
+
 static LISP
 im_set_encoding(LISP id, LISP enc)
 {
@@ -299,7 +325,8 @@ im_update_prop_list(LISP id, LISP prop_)
 {
   uim_context uc = uim_find_context(get_c_long(id));
   char *prop     = uim_get_c_string(prop_);
-  char *str;
+
+  printf("prop %s\n",prop);
 
   if(uc)
     uim_schedule_cb(uc, PROP_LIST_UPDATE_CB, NULL, 0, 0);
@@ -307,7 +334,7 @@ im_update_prop_list(LISP id, LISP prop_)
   if(uc->propstr)
     free(uc->propstr);
       
-  uc->propstr = uim_code_conv(prop, uc->conv);
+  uc->propstr = prop;//uim_code_conv(prop, uc->conv);
   
   return NIL;
 }
@@ -434,6 +461,7 @@ uim_init_im_subrs()
   /**/
   init_subr_2("im-commit",       im_commit);
   init_subr_1("im-commit-raw",   im_commit_raw);
+  init_subr_2("im-get-raw-key-str", im_get_raw_key_str);
   init_subr_2("im-set-encoding", im_set_encoding);
   /**/
   init_subr_3("im-register-im", im_register_im);
@@ -446,8 +474,8 @@ uim_init_im_subrs()
   init_subr_2("im-pushback-mode-list", im_pushback_mode_list);
   init_subr_1("im-update-mode-list",   im_update_mode_list);
   /**/
-  init_subr_1("im-update-prop-label", im_update_prop_label);
-  init_subr_1("im-update-prop-list",  im_update_prop_list);
+  init_subr_2("im-update-prop-label", im_update_prop_label);
+  init_subr_2("im-update-prop-list",  im_update_prop_list);
   /**/
   init_subr_2("im-update-mode", im_update_mode);
   /**/
