@@ -96,6 +96,9 @@ uim_create_context(void *ptr,
   /**/
   uc->mode_update_cb = NULL;
   /**/
+  uc->prop_list_update_cb  = NULL;
+  uc->prop_label_update_cb = NULL;
+  /**/
   uc->candidate_begin_cb = NULL;
   uc->candidate_update_cb = NULL;
   uc->candidate_end_cb = NULL;
@@ -177,7 +180,6 @@ void
 uim_set_prop_list_update_cb(uim_context uc,
 			    void (*update_cb)(void *ptr))
 {
-  printf("set prop list call %p\n",update_cb);
   uc->prop_list_update_cb = update_cb;
 }
 
